package gov.va.med.mhv.admin.service.impl.calendar;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import gov.va.med.mhv.admin.dto.CalendarCategoryDTO;
import gov.va.med.mhv.admin.dto.CalendarEventDTO;
import gov.va.med.mhv.admin.dto.CalendarSearchCriteriaDTO;
import gov.va.med.mhv.admin.dto.CategoryDTO;
import gov.va.med.mhv.admin.dto.MHVEventModel;
import gov.va.med.mhv.admin.dto.VisnDTO;
import gov.va.med.mhv.admin.service.calendar.CalendarCategoryService;
import gov.va.med.mhv.admin.service.calendar.CalendarEventService;
import gov.va.med.mhv.admin.service.calendar.CalendarService;
import gov.va.med.mhv.admin.service.calendar.VisnService;
import gov.va.med.mhv.admin.util.CommonUtility;
import gov.va.med.mhv.admin.util.MHVEventComparator;
import gov.va.med.mhv.admin.util.MHVEventTypeComparator;
import gov.va.med.mhv.common.api.exception.MHVException;

@Path("/")
@Service("calendarService")
@Component
public class CalendarServiceImpl implements CalendarService {

	private static Logger log = LogManager.getLogger(CalendarServiceImpl.class);
	private static final String PERSONAL_EVT = "Personal Events";
	private Date evtStartDate;
	private Date evtEndDate;

	@Autowired
	private CalendarEventService calendarEventService;
	
	@Autowired
	private VisnService visnService;
	
	@Autowired
	private CalendarCategoryService calendarCategoryService;


	@Override
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/findEventsForUser")
	public List<MHVEventModel> findEventsForUser(CalendarSearchCriteriaDTO searchCriteria) throws MHVException {
		List<MHVEventModel> events = new ArrayList<MHVEventModel>();
		List<String> eventTypes = searchCriteria.getEventTypes();
		if (searchCriteria.getKeyword() == null && (eventTypes == null || eventTypes.size() == 0)) {
			return events;
		}
		if (searchCriteria.getRangeStart() == null || searchCriteria.getRangeEnd() == null) {
			populateDefaultRanges(searchCriteria);
		}
		calculateDateRanges(searchCriteria);
		try {
			for (String eventType : eventTypes) {
				if (eventType == null || eventType.trim().length() == 0) {
					break;
				}
				getEvents(events, searchCriteria, eventType);
			}

			Collections.sort(events, new MHVEventComparator());

		} catch (MHVException e) {
			log.error(e);
			throw e;
		} catch (Exception e) {
			log.error(e);
			throw new MHVException(e);
		}
		return events;
	}

	@Override
	@POST
	@Consumes(MediaType.APPLICATION_JSON)
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/searchEventsForUser")
	public List<MHVEventModel> searchEventsForUser(CalendarSearchCriteriaDTO searchCriteria) throws MHVException {
		List<MHVEventModel> events = new ArrayList<MHVEventModel>();
		List<String> eventTypes = searchCriteria.getEventTypes();
		if (searchCriteria.getKeyword() == null && (eventTypes == null || eventTypes.size() == 0)) {
			return events;
		}
		if (searchCriteria.getRangeStart() == null || searchCriteria.getRangeEnd() == null) {
			populateDefaultRanges(searchCriteria);
		}
		calculateDateRanges(searchCriteria);

		try {
			
			for (String eventType : eventTypes) {
				if (eventType == null || eventType.trim().length() == 0) {
					break;
				}
				getEvents(events, searchCriteria, eventType);
			}

			//Collections.sort(events, new MHVEventComparator());

		} catch (MHVException e) {
			log.error(e);
			throw e;
		} catch (Exception e) {
			log.error(e);
			throw new MHVException(e);
		}
		return events;
	}

	private void calculateDateRanges(CalendarSearchCriteriaDTO searchCriteria) {
		String strStrDate = CommonUtility.dateToString(searchCriteria.getRangeStart(), "MM/dd/yyyy");
		String strEndDate = CommonUtility.dateToString(searchCriteria.getRangeEnd(), "MM/dd/yyyy");
		Date stDate = CommonUtility.stringToDate(strStrDate, "MM/dd/yyyy");
		Date endDate = CommonUtility.stringToDate(strEndDate, "MM/dd/yyyy");
		evtStartDate = CommonUtility.getDaTeTime(stDate, "0", "0");
		evtEndDate = CommonUtility.getDaTeTime(endDate, "23", "59");
	}

	private void getEvents(List<MHVEventModel> events, CalendarSearchCriteriaDTO searchCriteria,
			String eventType) throws MHVException {
		List<CalendarEventDTO> eventDtos = findEvents(searchCriteria, eventType);
		if (eventDtos != null) {
			List<MHVEventModel> allEvents = loadEvents(eventDtos, eventType, searchCriteria.getViewType());
			for (MHVEventModel mhvevent : allEvents) {
				if (searchCriteria.getKeyword() != null) {
					events.add(mhvevent);
				} else {
					if (mhvevent.getStartDate().after(evtEndDate) || mhvevent.getStartDate().before(evtStartDate)) {
					} else {
						events.add(mhvevent);
					}
				}
			}
		}
	}

	private List<CalendarEventDTO> findEvents(CalendarSearchCriteriaDTO searchCriteria, String eventType)
			throws MHVException {
		List<CalendarEventDTO> eventList = null;
		try {
			searchCriteria.setSelectedGenericEvent(eventType);
			eventList = calendarEventService.getEventsForAdminUser(searchCriteria);
		} catch (MHVException e) {
			throw e;
		} catch (Exception e) {
			log.error("Error in Find generic Events", e);
			throw new MHVException(e);
		}
		return eventList;
	}


	private List<MHVEventModel> loadEvents(List<CalendarEventDTO> calEvents, String evtType, String viewType) {

		List<MHVEventModel> events = new ArrayList<MHVEventModel>();

		for (CalendarEventDTO calEvent : calEvents) {
			MHVEventModel event = new MHVEventModel();
			event.setStartDate(calEvent.getStartDate());
			event.setEndDate(calEvent.getEndDate());
			event.setBeginDate(calEvent.getStartDate());
			event.setFinishDate(calEvent.getEndDate());
			event.setLocation(calEvent.getLocation());
			event.setEventType(evtType);
			event.setTitle(calEvent.getSubject());
			event.setEventName(calEvent.getSubject());
			event.setEventId(calEvent.getEventId());
			event.setDescription(calEvent.getDescription());
			event.setAllDay(calEvent.getAllDay());
			event.setRepeatInterval(calEvent.getRecurrenceInterval());
			event.setRepeatEndDate(calEvent.getRecurUntilDate());
			
			event.setTimeZone(calEvent.getTimeZone());
			event.setLink(calEvent.getLink());
			event.setVisnId(calEvent.getVisn().getVisnId());
			event.setCategoryId(calEvent.getCalendarCategory().getCalendarCategoryId());
			
			event.setVisnName(calEvent.getVisn().getVisnName());
			event.setCategoryName(calEvent.getCalendarCategory().getCategoryName());
			
			
			boolean multidayEvent = IsMultiDayEvent(calEvent);
			if (isEventEligible(calEvent.getRecurrenceInterval(), calEvent.getRecurUntilDate(),
					calEvent.getStartDate())) {
				events.add(event);
			}
			if ((calEvent.getRecurrenceInterval() != null && calEvent.getRecurUntilDate() != null) || (multidayEvent)) {
				if (multidayEvent) {
					event.setRepeatEndDate(CommonUtility.getDaTeTime(calEvent.getEndDate(), "0", "0"));
					event.setRepeatInterval("DAILY");
				}
				event.setRepeatEndDate(CommonUtility.getDaTeTime(event.getRepeatEndDate(), "0", "0"));
				List<MHVEventModel> recurEvents = getRecurringEvents(event, evtType, multidayEvent);
				for (MHVEventModel recurEvent : recurEvents) {
					if (multidayEvent) {
						String startDate = CommonUtility.dateToString(recurEvent.getStartDate(), "MM/dd/yyyy");
						String firstDate = CommonUtility.dateToString(event.getStartDate(), "MM/dd/yyyy");
						String lastDate = CommonUtility.dateToString(event.getEndDate(), "MM/dd/yyyy");

						if (!startDate.equals(firstDate) && !startDate.equals(lastDate)) {
							recurEvent.setStartDate(CommonUtility.getDaTeTime(recurEvent.getStartDate(), "0", "0"));
							recurEvent.setEndDate(CommonUtility.getDaTeTime(recurEvent.getStartDate(), "23", "59"));
						} else if (startDate.equals(lastDate)) {
							recurEvent.setStartDate(CommonUtility.getDaTeTime(recurEvent.getStartDate(), "0", "0"));
							recurEvent.setEndDate(CommonUtility.getDaTeTime(recurEvent.getStartDate(),
									CommonUtility.dateToString(event.getEndDate(), "HH"),
									CommonUtility.dateToString(event.getEndDate(), "mm")));

						}
					}

					events.add(recurEvent);
				}
				if (multidayEvent) {
					event.setRepeatEndDate(null);
					event.setRepeatInterval(null);
					event.setEndDate(CommonUtility.getDaTeTime(event.getStartDate(), "23", "59"));
				}

			}
		}
		return events;
	}

	private boolean isEventEligible(String recurInterval, Date recurUntilDate, Date startDate) {
		boolean eligible = false;
		if ((recurInterval != null && recurUntilDate != null)) {
			Calendar cal = Calendar.getInstance();
			cal.setTime(CommonUtility.getDaTeTime(startDate, "0", "0"));
			if (recurInterval.equals("MWF")) {
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) {

					eligible = true;
				}
			} else if (recurInterval.equals("TTH")) {
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY) {

					eligible = true;
				}
			} else if (recurInterval.equals("MTWThF")) {
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) {

					eligible = true;
				}
			} else {
				eligible = true;
			}
		} else {
			eligible = true;
		}
		return eligible;
	}

	private boolean IsMultiDayEvent(CalendarEventDTO calEvent) {
		boolean multiDayEvt = false;
		if (calEvent.getRecurrenceInterval() == null && calEvent.getRecurUntilDate() == null) {
			multiDayEvt = CommonUtility.dateToString(calEvent.getStartDate(), "MM/dd/yyyy")
					.equals(CommonUtility.dateToString(calEvent.getEndDate(), "MM/dd/yyyy")) ? false : true;
		}
		return multiDayEvt;
	}

	private void populateDefaultRanges(CalendarSearchCriteriaDTO searchCriteria) {
		Calendar cal = Calendar.getInstance();
		cal.set(Calendar.HOUR, 0);
		cal.set(Calendar.MINUTE, 0);
		cal.set(Calendar.SECOND, 0);
		cal.add(Calendar.YEAR, 10);
		searchCriteria.setRangeEnd(cal.getTime());
		cal.add(Calendar.YEAR, -20);
		cal.set(Calendar.HOUR, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);

		searchCriteria.setRangeStart(cal.getTime());
	}

	private List<MHVEventModel> getRecurringEvents(MHVEventModel event, String eventType, boolean multiDayEvent) {
		List<MHVEventModel> recurEvents = new ArrayList<MHVEventModel>();
		Date recurEndDate = event.getRepeatEndDate();
		Calendar cal = Calendar.getInstance();
		cal.setTime(recurEndDate);
		cal.set(Calendar.HOUR, 23);
		cal.set(Calendar.MINUTE, 59);
		cal.set(Calendar.SECOND, 59);
		Date endDate = cal.getTime();
		Date beginDate = event.getStartDate();
		cal.setTime(beginDate);
		Date startDate = cal.getTime();
		Date tempDate = cal.getTime();

		Calendar calEnd = Calendar.getInstance();
		calEnd.setTime(event.getEndDate());

		for (;;) {
			if (event.getRepeatInterval().equals("DAILY")) {
				cal.setTime(tempDate);
				cal.add(Calendar.DAY_OF_YEAR, 1);
				calEnd.add(Calendar.DAY_OF_YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
				newevent.setStartDate(nextDate);
				newevent.setEndDate(nextEndDate);
				tempDate = nextDate;
				recurEvents.add(newevent);
			} else if (event.getRepeatInterval().equals("MWF")) {
				cal.setTime(tempDate);
				cal.add(Calendar.DAY_OF_YEAR, 1);
				calEnd.add(Calendar.DAY_OF_YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) {
					MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
					newevent.setStartDate(nextDate);
					newevent.setEndDate(nextEndDate);
					recurEvents.add(newevent);
				}
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("TTH")) {
				cal.setTime(tempDate);
				cal.add(Calendar.DAY_OF_YEAR, 1);
				calEnd.add(Calendar.DAY_OF_YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY) {
					MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
					newevent.setStartDate(nextDate);
					newevent.setEndDate(nextEndDate);
					recurEvents.add(newevent);
				}
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("MTWThF")) {
				cal.setTime(tempDate);
				cal.add(Calendar.DAY_OF_YEAR, 1);
				calEnd.add(Calendar.DAY_OF_YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				if (cal.get(Calendar.DAY_OF_WEEK) == Calendar.MONDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.TUESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.WEDNESDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.THURSDAY
						|| cal.get(Calendar.DAY_OF_WEEK) == Calendar.FRIDAY) {
					MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
					newevent.setStartDate(nextDate);
					newevent.setEndDate(nextEndDate);
					recurEvents.add(newevent);
				}
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("MONTHLY")) {
				cal.setTime(tempDate);
				cal.add(Calendar.MONTH, 1);
				calEnd.add(Calendar.MONTH, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
				newevent.setStartDate(nextDate);
				newevent.setEndDate(nextEndDate);
				recurEvents.add(newevent);
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("WEEKLY")) {
				cal.setTime(tempDate);
				cal.add(Calendar.WEEK_OF_YEAR, 1);
				calEnd.add(Calendar.WEEK_OF_YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
				newevent.setStartDate(nextDate);
				newevent.setEndDate(nextEndDate);
				recurEvents.add(newevent);
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("ALTWEEK")) {
				cal.setTime(tempDate);
				cal.add(Calendar.WEEK_OF_YEAR, 2);
				calEnd.add(Calendar.WEEK_OF_YEAR, 2);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
				newevent.setStartDate(nextDate);
				newevent.setEndDate(nextEndDate);
				recurEvents.add(newevent);
				tempDate = nextDate;

			} else if (event.getRepeatInterval().equals("YEARLY")) {
				cal.setTime(tempDate);
				cal.add(Calendar.YEAR, 1);
				calEnd.add(Calendar.YEAR, 1);
				Date nextDate = cal.getTime();
				Date nextEndDate = calEnd.getTime();
				if (nextDate.after(endDate)) {
					break;
				}
				MHVEventModel newevent = copyEvent(event, eventType, multiDayEvent);
				newevent.setStartDate(nextDate);
				newevent.setEndDate(nextEndDate);
				tempDate = nextDate;
				recurEvents.add(newevent);
			} else {
				break;
			}
		}
		return recurEvents;
	}

	private MHVEventModel copyEvent(MHVEventModel event, String eventType, boolean multiDayEvent) {
		MHVEventModel newevent = new MHVEventModel();
		newevent.setStartDate(event.getStartDate());
		newevent.setEndDate(event.getEndDate());
		newevent.setBeginDate(event.getStartDate());
		if (!eventType.equals("My Tasks")) {
			newevent.setFinishDate(event.getEndDate());
		} else {
			newevent.setFinishDate(event.getFinishDate());
		}

		newevent.setLocation(event.getLocation());
		newevent.setEventType(eventType);
		newevent.setTitle(event.getTitle());
		newevent.setEventName(event.getEventName());
		newevent.setEventId(event.getEventId());
		newevent.setDescription(event.getDescription());
		newevent.setAllDay(event.isAllDay());
		if (!multiDayEvent) {
			newevent.setRepeatInterval(event.getRepeatInterval());
			newevent.setRepeatEndDate(event.getRepeatEndDate());
		}
		
		newevent.setTimeZone(event.getTimeZone());
		newevent.setLink(event.getLink());
		newevent.setVisnId(event.getVisnId());
		newevent.setCategoryId(event.getCategoryId());
		
		newevent.setVisnName(event.getVisnName());
		newevent.setCategoryName(event.getCategoryName());
		
		
		return newevent;
	}

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/getAllCategories")
	public List<CategoryDTO> getAllCategories() throws MHVException {
		List<CategoryDTO> categories = null;
		try {
			categories = calendarEventService.getAllCategories();
		} catch (MHVException e) {
			throw e;
		}
		return categories;
	}

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("getAllActiveCategories/{userProfileId}")
	public List<CalendarCategoryDTO> getAllActiveCategories(@PathParam("userProfileId") Long userProfileId)
			throws MHVException {
		List<CalendarCategoryDTO> categories = null;
		try {
			categories = calendarEventService.getActiveCategories(userProfileId);
		} catch (MHVException e) {
			throw e;
		}
		return categories;
	}

	@Override
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("/saveCalendarEvent")
	public void saveCalendarEvent(CalendarEventDTO calendarEventDto) throws MHVException {
		try {
			calendarEventService.saveCalendarEvent(calendarEventDto);
		} catch (MHVException e) {
			throw e;
		}

	}

	@Override
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/deleteCalendarEvent/{calendarEventId}")
	public void deleteCalendarEvent(@PathParam("calendarEventId") Long calendarEventId) throws MHVException {
		try {
			calendarEventService.deleteCalendarEvent(calendarEventId);
		} catch (MHVException e) {
			throw e;
		}
	}

	@Override
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("/saveUserCalCatList/{userProfileId}")
	public void saveUserCalCatList(List<CategoryDTO> userCalCatList, @PathParam("userProfileId") Long useProfileId)
			throws MHVException {
		try {
			calendarEventService.saveUserCalCatList(userCalCatList, useProfileId);
		} catch (MHVException e) {
			throw e;
		}

	}

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/getCheckedCategories/{userProfileId}")
	public List<CategoryDTO> getCheckedCategories(@PathParam("userProfileId") Long userProfileId) throws MHVException {
		List<CategoryDTO> categories = null;
		try {
			categories = calendarEventService.getCheckedCategories(userProfileId);
			if (categories != null) {
				Collections.sort(categories, new MHVEventTypeComparator());
			}
		} catch (MHVException e) {
			throw e;
		}
		return categories;
	}

	@Override
	@GET
	@Produces(MediaType.WILDCARD)
	@Consumes(MediaType.WILDCARD)
	@Path("/external/stream/report/{rangeStart}/{rangeEnd}/{eventTypes}")
	public Response generateReport(@PathParam("rangeStart") String rangeStart, @PathParam("rangeEnd") String rangeEnd,
			@PathParam("eventTypes") String eventTypes) {
		try {
			List<MHVEventModel> mhvEvents = null;
			String[] evtTypes = eventTypes.split(",");
			List<String> eventTypeList = new ArrayList<String>();
			List<CategoryDTO> calCatgs = getAllCategories();
			
			for (String evtType : evtTypes) {
				if(evtType != null){
					for(CategoryDTO catg: calCatgs){
						
						if(Long.valueOf(evtType).longValue()== catg.getCategoryId().longValue()){
							eventTypeList.add(catg.getCategory());
							break;
						}
						
					}
				}
			}
			
			
			/*Date exportStartDate = CommonUtility.stringToDate(rangeStart, "MMddyyyy");
			Date exportEndDate = CommonUtility.stringToDate(rangeEnd, "MMddyyyy");*/
			
			Date exportStartDate = CommonUtility.getDaTeTime(CommonUtility.stringToDate(rangeStart, "MMddyyyy"), "0", "0");
			Date exportEndDate = CommonUtility.getDaTeTime(CommonUtility.stringToDate(rangeEnd, "MMddyyyy"), "23", "59");
			
			
			CalendarSearchCriteriaDTO searchCriteria = new CalendarSearchCriteriaDTO();
			searchCriteria.setRangeStart(exportStartDate);
			searchCriteria.setRangeEnd(exportEndDate);
			searchCriteria.setEventTypes(eventTypeList);
			searchCriteria.setViewType("listView");
			mhvEvents = findEventsForUser(searchCriteria);
			String result = buildCSVData(mhvEvents);
			String fileName = "admin_calendar.csv";
			return Response.ok().entity(result.getBytes()).header("Content-Type", "application/vnd.ms-excel")
					.header("Content-Length", result.getBytes().length)
					.header("Content-Disposition", "attachment; filename=" + fileName).build();

		} catch (MHVException ex) {
			log.error("Generate Report: ", ex);
			return Response.status(400).build();
		}
	}

	private String buildCSVData(List<MHVEventModel> mhvEvents) {
		StringBuffer sb = new StringBuffer();
		sb.append("Subject");
		sb.append(",");
		sb.append("Start Date");
		sb.append(",");
		sb.append("Start Time");
		sb.append(",");
		sb.append("End Date");
		sb.append(",");
		sb.append("End Time");
		sb.append(",");
		sb.append("All day event");
		sb.append(",");
		sb.append("Reminder on/off");
		sb.append(",");
		sb.append("Reminder Date");
		sb.append(",");
		sb.append("Reminder Time");
		sb.append(",");
		sb.append("Categories");
		sb.append(",");
		sb.append("Description");
		sb.append(",");
		sb.append("Location");
		sb.append("\n");

		for (MHVEventModel event : mhvEvents) {
			sb.append(event.getEventName());
			sb.append(",");
			sb.append(CommonUtility.dateToString(event.getStartDate(), "MM/dd/yyy"));
			sb.append(",");
			sb.append(CommonUtility.dateToString(event.getStartDate(), "h:mm a"));
			sb.append(",");
			sb.append(CommonUtility.dateToString(event.getEndDate(), "MM/dd/yyy"));
			sb.append(",");
			sb.append(CommonUtility.dateToString(event.getEndDate(), "h:mm a"));
			sb.append(",");
			sb.append(event.isAllDay());
			sb.append(",");
			sb.append(" ");
			sb.append(",");
			sb.append(" ");
			sb.append(",");
			sb.append(" ");
			sb.append(",");
			sb.append(event.getEventType());
			sb.append(",");
			sb.append(event.getDescription() != null ? event.getDescription() : " ");
			sb.append(",");
			sb.append(event.getLocation() != null ? event.getLocation() : " ");
			sb.append("\n");
		}

		return sb.toString();
	}

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/personalEventById/{eventId}")
	public MHVEventModel getPersonalEventById(@PathParam("eventId") Long eventId) throws MHVException {
		CalendarEventDTO calEvent = calendarEventService.getPersonalEventById(eventId);

		MHVEventModel event = new MHVEventModel();
		event.setStartDate(calEvent.getStartDate());
		event.setEndDate(calEvent.getEndDate());
		event.setBeginDate(calEvent.getStartDate());
		event.setFinishDate(calEvent.getEndDate());
		event.setLocation(calEvent.getLocation());
		event.setEventType(PERSONAL_EVT);
		event.setTitle(calEvent.getSubject());
		event.setEventName(calEvent.getSubject());
		event.setEventId(calEvent.getEventId());
		event.setDescription(calEvent.getDescription());
		event.setAllDay(calEvent.getAllDay());
		event.setRepeatInterval(calEvent.getRecurrenceInterval());
		event.setRepeatEndDate(calEvent.getRecurUntilDate());
		
		event.setCategoryId(calEvent.getCalendarCategory().getCalendarCategoryId());
		event.setTimeZone(calEvent.getTimeZone());
		event.setVisnId(calEvent.getVisn().getVisnId());
		event.setLink(calEvent.getLink());
		
		event.setVisnName(calEvent.getVisn().getVisnName());
		event.setCategoryName(calEvent.getCalendarCategory().getCategoryName());
		
		return event;
	}

	public Date getEvtStartDate() {
		return evtStartDate;
	}

	public void setEvtStartDate(Date evtStartDate) {
		this.evtStartDate = evtStartDate;
	}

	public Date getEvtEndDate() {
		return evtEndDate;
	}

	public void setEvtEndDate(Date evtEndDate) {
		this.evtEndDate = evtEndDate;
	}

	@Override
	@GET
	@Produces(MediaType.APPLICATION_JSON)
	@Path("/getAllVisns")
	public List<VisnDTO> getAllVisns() throws MHVException {
		List<VisnDTO> visns=null;
		try{
			visns=visnService.getAllVisns();
		}catch(Exception e){
			throw new MHVException(e);
		}
		return visns;
	}

	@Override
	@POST
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("/saveCalendarCategory")
	public void saveCalendarCategory(CalendarCategoryDTO categoryDTO) throws MHVException {
		try{
			 calendarCategoryService.saveCalendarCategory(categoryDTO);
		}catch(Exception e){
			throw e;
		}
	}

	@Override
	@DELETE
	@Produces(MediaType.APPLICATION_JSON)
	@Consumes(MediaType.APPLICATION_JSON)
	@Path("/deleteCalendarCategory/{categoryId}")
	public void deleteCalendarCategory(@PathParam("categoryId")  Long categoryId) throws MHVException {
		try{
			 calendarCategoryService.deleteCalendarCategory(categoryId);
		}catch(Exception e){
			throw new MHVException(e);
		}
	}
}
