package gov.va.med.mhv.admin.web.controller;

import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.BeansException;
import org.apache.commons.lang.StringUtils;

import javax.portlet.PortletRequest;
import javax.portlet.PortletSession;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import com.liferay.portal.kernel.exception.PortalException;
import com.liferay.portal.kernel.exception.SystemException;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.model.User;


import gov.va.med.mhv.admin.dto.EmployeeDTO;
import gov.va.med.mhv.admin.dto.EmployeeOrgRoleDTO;
import gov.va.med.mhv.admin.dto.EmployeeSearchResult;
import gov.va.med.mhv.admin.service.vastaff.EmployeeRoleManagementService;
import gov.va.med.mhv.admin.service.vastaff.EmployeeSearchService;
import gov.va.med.mhv.admin.web.formbean.SearchResultsFormBean;
import gov.va.med.mhv.admin.web.formbean.StaffSearchFormBean;
import gov.va.med.mhv.admin.web.util.SessionUtil;
import gov.va.med.mhv.admin.web.util.SortUtil;
import gov.va.med.mhv.admin.web.util.SpringContext;
import gov.va.med.mhv.common.api.exception.MHVException;

@ManagedBean(name = "searchController")
@RequestScoped
public class SearchStaffController {

	private static Logger log = LogManager.getLogger(SearchStaffController.class);

	protected static final String ERR_PRCS_RQST = "Error Processing request";

	@ManagedProperty(value = "#{search}")
	StaffSearchFormBean searchForm;

	@ManagedProperty(value = "#{staffSearchResults}")
	SearchResultsFormBean searchResults;

	@ManagedProperty(value = "#{sessionUtil}")
	SessionUtil sessionUtil;

	/**
	 * Invoked when a user clicks the "Search" button on the search form UI.
	 * 
	 * @return the view to advance to, whether search results or back to search
	 *         form.
	 */
	public String search() {
		String result = "";
		searchResults.setCameFrom(SearchResultsFormBean.CameFrom.SEARCH);
		String id = searchForm.getVaNetworkId();
		try {
			EmployeeSearchService employeeSearchService = (EmployeeSearchService) SpringContext.getApplicationContext()
					.getBean("employeeSearchServiceProxy");
			List<EmployeeSearchResult> employees = null;
			String lastName = searchForm.getLastName();
			if (lastName != null && !"".equals(lastName.trim())) {
				// search by name
				String firstName = searchForm.getFirstName();
				if (firstName == null || "".equals(firstName)) {
					firstName = " ";
				}
				employees = employeeSearchService.searchByName(firstName, lastName);
			} else {

				employees = employeeSearchService.searchByUserName(id);

			}
			if (employees != null) {
				// sort
				new SortUtil().imposeDefaultSortOrderSearch(employees);

				// stick employee in flash for redirect to search results page
				searchResults.setEmployees(employees);
				result = "search-staff-results?faces-redirect=true";
			} else {
				// stay on search criteria page
				FacesContext.getCurrentInstance().addMessage("No search results.",
						new FacesMessage("Please check your search criteria and try again."));
			}

		} catch (MHVException e) {
			log.error("Error in SearchStaffController.search() - Searching Proxy 'employeeSearchServiceProxy'", e);
			processErrorMessages(e);
		} catch (Exception e) {
			log.error("General error in SearchStaffController.search() - Searching Proxy 'employeeSearchServiceProxy'",
					e);
			FacesContext.getCurrentInstance().addMessage("Error Occurred",
					new FacesMessage("A general error occurred performing Staff Search, please retry."));
		}
		return result;
	}

	public String viewAll() {

		String result = "";
		searchResults.setCameFrom(SearchResultsFormBean.CameFrom.VIEW_ALL);
		try {
			EmployeeRoleManagementService service = (EmployeeRoleManagementService) SpringContext
					.getApplicationContext().getBean("employeeRoleManagementServiceProxy");
			
			String loggedInuserName = getAdminUserName();
			EmployeeDTO loggedInEmployee = sessionUtil.getEmployee(loggedInuserName);

			//EmployeeDTO loggedInEmployee = sessionUtil.getCurrentUser();
			if(loggedInEmployee != null) {
				List<EmployeeOrgRoleDTO> assignedRoles = service
						.getEmployeeOrganizationRolesEmployeeCanManage(loggedInEmployee.getEmployeeId());
	
				// impose default sort order: Sory by Name then by Organization (so
				// sort in reverse order, logically)
				new SortUtil().imposeDefaultSortOrderViewAll(assignedRoles);
	
				searchResults.setRoles(assignedRoles);
			
			}

			result = "view-all";
		} catch (BeansException e) {
			log.error(
					"Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
					e);
			FacesContext.getCurrentInstance().addMessage("Error Occurred", new FacesMessage(
					"An error occurred retrieving the assigned Employee Organizational Roles, please retry."));
		} catch (MHVException e) {
			log.error(
					"Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
					e);
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_ERROR, e.getFailureMessage(), null));
		} catch (RuntimeException e) {
			log.error(
					"Unexpected Error in employeeRoleManagementServiceProxy.getEmployeeOrgRolesAssignedToEmployeeByName() - Searching Proxy 'employeeRoleManagementServiceProxy'",
					e);
			FacesContext.getCurrentInstance().addMessage("Unexpected Error Occurred", new FacesMessage(
					"An error occurred retrieving the assigned Employee Organizational Roles, please retry."));
		}
		return result;
	}

	public String newSearch() {
		return "search-staff";
	}

	public StaffSearchFormBean getSearchForm() {
		return searchForm;
	}

	public void setSearchForm(StaffSearchFormBean searchForm) {
		this.searchForm = searchForm;
	}

	public SearchResultsFormBean getSearchResults() {
		return searchResults;
	}

	public void setSearchResults(SearchResultsFormBean searchResults) {
		this.searchResults = searchResults;
	}

	public String backToSearchResults() {
		String result = "";
		if (searchResults.getCameFrom() == SearchResultsFormBean.CameFrom.VIEW_ALL) {
			result = "view-all-current-roles?faces-redirect=true";
		} else {
			result = "search-staff-results?faces-redirect=true";
		}
		return result;
	}

	public SessionUtil getSessionUtil() {
		return sessionUtil;
	}

	public void setSessionUtil(SessionUtil sessionUtil) {
		this.sessionUtil = sessionUtil;
	}

	public void processErrorMessages(MHVException exp) {
		// response has errors
		if (null != exp.getErrorCode()) {
			Map<String, String> validationErrors = exp.getValidationErrorMessages();
			validationErrors.put("No VA staff members were found matching your search criteria.",
					"No VA staff members were found matching your search criteria.");

			if (log.isDebugEnabled()) {
				int size = validationErrors.size();
				log.debug("validationErrors.size() : " + size);
			}

			// check if there are any validation errors in response object
			if (validationErrors.size() > 0) {
				System.out.println("********************************************null != validationErrors ");
				for (Map.Entry<String, String> err : validationErrors.entrySet()) {
					if (log.isDebugEnabled()) {
						if (null != err) {
							log.debug("entry.getKey() : " + err.getKey());
							log.debug("entry.getValue() : " + err.getValue());
						}
					}
					if (null != err)
						FacesContext.getCurrentInstance().addMessage(null,
								new FacesMessage(FacesMessage.SEVERITY_ERROR, err.getValue(), err.getValue()));
				}
			}

			// check for non-validation errors in response object
			if (null != exp.getFailureMessage() && exp.getFailureMessage().length() > 0) {
				if (log.isDebugEnabled()) {
					log.debug("FailureMessage : " + exp.getFailureMessage());
				}

				// FacesContext.getCurrentInstance().addMessage(null, new
				// FacesMessage(FacesMessage.SEVERITY_ERROR, ERR_PRCS_RQST,
				// ERR_PRCS_RQST));
			}
		}
	}
	
	public String getAdminUserName() {
		String adminUserNameStr = "";
		User liferayUser = null;
		PortletRequest request = (PortletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
		HttpServletRequest httpRequest = PortalUtil.getHttpServletRequest(request);
		HttpSession session = httpRequest.getSession();
		adminUserNameStr = (String) session.getAttribute("LIFERAY_SHARED_screenName");
		if (StringUtils.isBlank(adminUserNameStr)) {
			try {
				liferayUser = PortalUtil.getUser(request);
				if (liferayUser != null) {
					adminUserNameStr = liferayUser.getScreenName();
				}
			} catch (PortalException e) {
				e.printStackTrace();
			} catch (SystemException e) {
				e.printStackTrace();
			}
		}
		return adminUserNameStr;
		// return "vacojonnag";
	}

}
